//---------------------------------------------------------------------------


#pragma hdrstop

#include "Settings.h"
#include <algorithm>
#include <fstream> 
#include <json/json.h>

//---------------------------------------------------------------------------

#pragma package(smart_init)

Settings appSettings;

inline void strncpyz(char* dst, const char* src, int dstsize) {
	strncpy(dst, src, dstsize);
	dst[dstsize-1] = '\0';
}

void Settings::SetDefault(void)
{
	frmMain.iWidth = 350;
	frmMain.iHeight = 300;
	frmMain.iPosX = 30;
	frmMain.iPosY = 30;
	frmMain.bWindowMaximized = false;
	frmMain.bAlwaysOnTop = false;

	Logging.bLogToFile = false;
	Logging.iMaxUiLogLines = 5000;

	uaConf.accounts.clear();

	struct UaConf::Account new_acc;
	uaConf.accounts.push_back(new_acc);
}

int Settings::Read(AnsiString asFileName)
{
	Json::Value root;   // will contains the root value after parsing.
	Json::Reader reader;

    SetDefault();

	try
	{
		std::ifstream ifs(asFileName.c_str());
		std::string strConfig((std::istreambuf_iterator<char>(ifs)), std::istreambuf_iterator<char>());
		ifs.close();
		bool parsingSuccessful = reader.parse( strConfig, root );
		if ( !parsingSuccessful )
		{
			return 2;
		}
	}
	catch(...)
	{
		return 1;
	}

	const Json::Value &uaConfJson = root["uaConf"];
	const Json::Value &uaConfAudioCfgSrcJson = uaConfJson["audioCfgSrc"];
	strncpyz(uaConf.audioCfgSrc.mod, uaConfAudioCfgSrcJson.get("mod", UaConf::modWinwave).asString().c_str(), sizeof(uaConf.audioCfgSrc.mod));
	strncpyz(uaConf.audioCfgSrc.dev, uaConfAudioCfgSrcJson["dev"].asString().c_str(), sizeof(uaConf.audioCfgSrc.dev));
	if (strcmp(uaConf.audioCfgSrc.mod, UaConf::modPortaudio) &&
		strcmp(uaConf.audioCfgSrc.mod, UaConf::modWinwave)) {
		strncpyz(uaConf.audioCfgSrc.mod, UaConf::modWinwave, sizeof(uaConf.audioCfgSrc.mod));
	}
	const Json::Value &uaConfAudioCfgPlayJson = uaConfJson["audioCfgPlay"];
	strncpyz(uaConf.audioCfgPlay.mod, uaConfAudioCfgPlayJson.get("mod", UaConf::modWinwave).asString().c_str(), sizeof(uaConf.audioCfgPlay.mod));
	strncpyz(uaConf.audioCfgPlay.dev, uaConfAudioCfgPlayJson["dev"].asString().c_str(), sizeof(uaConf.audioCfgPlay.dev));
	if (strcmp(uaConf.audioCfgPlay.mod, UaConf::modPortaudio) &&
		strcmp(uaConf.audioCfgPlay.mod, UaConf::modWinwave)) {
		strncpyz(uaConf.audioCfgPlay.mod, UaConf::modWinwave, sizeof(uaConf.audioCfgPlay.mod));
	}
	const Json::Value &uaConfAudioCfgAlertJson = uaConfJson["audioCfgAlert"];
	strncpyz(uaConf.audioCfgAlert.mod, uaConfAudioCfgAlertJson.get("mod", UaConf::modWinwave).asString().c_str(), sizeof(uaConf.audioCfgAlert.mod));
	strncpyz(uaConf.audioCfgAlert.dev, uaConfAudioCfgAlertJson["dev"].asString().c_str(), sizeof(uaConf.audioCfgAlert.dev));
	if (strcmp(uaConf.audioCfgAlert.mod, UaConf::modPortaudio) &&
		strcmp(uaConf.audioCfgAlert.mod, UaConf::modWinwave)) {
		strncpyz(uaConf.audioCfgAlert.mod, UaConf::modWinwave, sizeof(uaConf.audioCfgAlert.mod));
	}

	uaConf.aec = (enum UaConf::Aec)uaConfJson.get("aec", UaConf::AEC_WEBRTC).asInt();
	if (uaConf.aec < 0 || uaConf.aec >= UaConf::AEC_LIMIT)
	{
    	uaConf.aec = UaConf::AEC_WEBRTC;
	}

	const Json::Value &webrtcAec = root["webrtcAec"];
	uaConf.webrtcAec.msInSndCardBuf = webrtcAec.get("msInSndCardBuf", 120).asInt();
	uaConf.webrtcAec.skew = webrtcAec.get("skew", 0).asInt();

	uaConf.logMessages = uaConfJson.get("logMessages", false).asBool();
	uaConf.local = uaConfJson.get("localAddress", "").asString();
	uaConf.autoAnswer = uaConfJson.get("autoAnswer", false).asBool();
	uaConf.autoAnswerCode = uaConfJson.get("autoAnswerCode", 200).asInt();

	uaConf.codecs.g711 = uaConfJson["Codecs"].get("G.711", true).asBool();
	uaConf.codecs.g726_32 = uaConfJson["Codecs"].get("G.726-32", false).asBool();
	uaConf.codecs.gsm = uaConfJson["Codecs"].get("GSM", false).asBool();
	uaConf.codecs.speex = uaConfJson["Codecs"].get("Speex", false).asBool();

	const Json::Value &accounts = root["Accounts"];
	//uaConf.accounts.clear();
	for (int i=0; i<std::min(1u, accounts.size()); i++)
	{
		const Json::Value &acc = accounts[i];
		struct UaConf::Account new_acc;

		new_acc.reg_server = acc.get("reg_server", "").asString();
		new_acc.user = acc.get("user", "").asString();
		new_acc.auth_user = acc.get("auth_user", "").asString();
		new_acc.pwd = acc.get("pwd", "").asString();
		new_acc.transport =
			(UaConf::Account::TRANSPORT_TYPE)acc.get("transport", UaConf::Account::TRANSPORT_UDP).asInt();
		if (new_acc.transport < 0 || new_acc.transport >= UaConf::Account::TRANSPORT_LIMITER)
		{
			new_acc.transport = UaConf::Account::TRANSPORT_UDP;
		}
		new_acc.reg_expires = acc.get("reg_expires", 60).asInt();
		if (new_acc.reg_expires < 0 || new_acc.reg_expires > 7200)
		{
			new_acc.reg_expires = 60;
		}

		new_acc.stun_server = acc.get("stun_server", "").asString();

		uaConf.accounts[0] = new_acc;
	}

	const Json::Value &contacts = root["Contacts"];
	uaConf.contacts.clear();
	for (int i=0; i<contacts.size(); i++)
	{
		const Json::Value &contact = contacts[i];
		struct UaConf::Contact new_contact;

		new_contact.description = contact.get("description", "").asString();
		new_contact.user = contact.get("user", "").asString();
		new_contact.sub_dialog_info = contact.get("sub_dialog_info", false).asBool();
		uaConf.contacts.push_back(new_contact);
	}

	for (int i=contacts.size(); i<UaConf::CONTACTS_CNT; i++)
	{
		struct UaConf::Contact new_contact;
		uaConf.contacts.push_back(new_contact);
	}

	int maxX = GetSystemMetrics(SM_CXSCREEN);
	/** \todo Ugly fixed taskbar margin */
	int maxY = GetSystemMetrics(SM_CYSCREEN) - 32;

	const Json::Value &frmMainJson = root["frmMain"];
	frmMain.iWidth = frmMainJson.get("AppWidth", 350).asInt();
	frmMain.iHeight = frmMainJson.get("AppHeight", 300).asInt();
	if (frmMain.iWidth < 250 || frmMain.iWidth > maxX + 20)
	{
		frmMain.iWidth = 250;
	}
	if (frmMain.iHeight < 200 || frmMain.iHeight > maxY + 20)
	{
		frmMain.iHeight = 200;
	}
	frmMain.iPosX = frmMainJson.get("AppPositionX", 30).asInt();
	frmMain.iPosY = frmMainJson.get("AppPositionY", 30).asInt();
	if (frmMain.iPosX < 0)
		frmMain.iPosX = 0;
	if (frmMain.iPosY < 0)
		frmMain.iPosY = 0;
	if (frmMain.iPosX + frmMain.iWidth > maxX)
		frmMain.iPosX = maxX - frmMain.iWidth;
	if (frmMain.iPosY + frmMain.iHeight > maxY)
		frmMain.iPosY = maxY - frmMain.iHeight;
	frmMain.bWindowMaximized = frmMainJson.get("Maximized", false).asBool();
	frmMain.bAlwaysOnTop = frmMainJson.get("AlwaysOnTop", false).asBool();
	frmMain.bSpeedDialVisible = frmMainJson.get("SpeedDialVisible", false).asBool();
	frmMain.bStartMinimizedToTray = frmMainJson.get("StartMinimizedToTray", false).asBool();

	const Json::Value &frmTrayNotifierJson = root["frmTrayNotifier"];
	frmTrayNotifier.iHeight = 105;
	frmTrayNotifier.iWidth = 213;
	frmTrayNotifier.iPosX = frmTrayNotifierJson.get("PosX", maxX - frmTrayNotifier.iWidth).asInt();
	frmTrayNotifier.iPosY = frmTrayNotifierJson.get("PosY", maxY - frmTrayNotifier.iHeight).asInt();
	if (frmTrayNotifier.iPosX < 0)
		frmTrayNotifier.iPosX = 0;
	if (frmTrayNotifier.iPosY < 0)
		frmTrayNotifier.iPosY = 0;
	if (frmTrayNotifier.iPosX + frmMain.iWidth > maxX)
		frmTrayNotifier.iPosX = maxX - frmTrayNotifier.iWidth;
	if (frmTrayNotifier.iPosY + frmMain.iHeight > maxY)
		frmTrayNotifier.iPosY = maxY - frmTrayNotifier.iHeight;
	frmTrayNotifier.showOnIncoming = frmTrayNotifierJson.get("ShowOnIncoming", false).asBool();
	frmTrayNotifier.skipIfMainWindowVisible = frmTrayNotifierJson.get("SkipIfMainWindowVisible", false).asBool();

	const Json::Value LoggingJson = root["Logging"];
	Logging.bLogToFile = LoggingJson.get("LogToFile", false).asBool();
	Logging.iMaxUiLogLines = LoggingJson.get("MaxUiLogLines", 5000).asInt();

	return 0;
}

int Settings::Write(AnsiString asFileName)
{
    Json::Value root;
	Json::StyledWriter writer;

	root["frmMain"]["AppWidth"] = frmMain.iWidth;
	root["frmMain"]["AppHeight"] = frmMain.iHeight;
	root["frmMain"]["AppPositionX"] = frmMain.iPosX;
	root["frmMain"]["AppPositionY"] = frmMain.iPosY;
	root["frmMain"]["Maximized"] = frmMain.bWindowMaximized;
	root["frmMain"]["AlwaysOnTop"] = frmMain.bAlwaysOnTop;
	root["frmMain"]["SpeedDialVisible"] = frmMain.bSpeedDialVisible;
	root["frmMain"]["StartMinimizedToTray"] = frmMain.bStartMinimizedToTray;

	root["frmTrayNotifier"]["PosX"] = frmTrayNotifier.iPosX;
	root["frmTrayNotifier"]["PosY"] = frmTrayNotifier.iPosY;
	root["frmTrayNotifier"]["ShowOnIncoming"] = frmTrayNotifier.showOnIncoming;
	root["frmTrayNotifier"]["SkipIfMainWindowVisible"] = frmTrayNotifier.skipIfMainWindowVisible;

	root["Logging"]["LogToFile"] = Logging.bLogToFile;
	root["Logging"]["MaxUiLogLines"] = Logging.iMaxUiLogLines;

	root["uaConf"]["audioCfgSrc"]["mod"] = uaConf.audioCfgSrc.mod;
	root["uaConf"]["audioCfgSrc"]["dev"] = uaConf.audioCfgSrc.dev;
	root["uaConf"]["audioCfgPlay"]["mod"] = uaConf.audioCfgPlay.mod;
	root["uaConf"]["audioCfgPlay"]["dev"] = uaConf.audioCfgPlay.dev;
	root["uaConf"]["audioCfgAlert"]["mod"] = uaConf.audioCfgAlert.mod;
	root["uaConf"]["audioCfgAlert"]["dev"] = uaConf.audioCfgAlert.dev;

	root["uaConf"]["aec"] = uaConf.aec;
	root["uaConf"]["logMessages"] = uaConf.logMessages;
	root["uaConf"]["localAddress"] = uaConf.local;
	root["uaConf"]["autoAnswer"] = uaConf.autoAnswer;
	root["uaConf"]["autoAnswerCode"] = uaConf.autoAnswerCode;

	root["uaConf"]["webrtcAec"]["msInSndCardBuf"] = uaConf.webrtcAec.msInSndCardBuf;
	root["uaConf"]["webrtcAec"]["skew"] = uaConf.webrtcAec.skew;

	root["uaConf"]["Codecs"]["G.711"] = uaConf.codecs.g711;
	root["uaConf"]["Codecs"]["G.726-32"] = uaConf.codecs.g726_32;
	root["uaConf"]["Codecs"]["GSM"] = uaConf.codecs.gsm;
	root["uaConf"]["Codecs"]["Speex"] = uaConf.codecs.speex;

	// write accounts
	for (unsigned int i=0; i<uaConf.accounts.size(); i++)
	{
		UaConf::Account &acc = uaConf.accounts[i];
		root["Accounts"][i]["reg_server"] = acc.reg_server;
		root["Accounts"][i]["user"] = acc.user;
		root["Accounts"][i]["auth_user"] = acc.auth_user;
		root["Accounts"][i]["pwd"] = acc.pwd;
		root["Accounts"][i]["transport"] = acc.transport;
		root["Accounts"][i]["reg_expires"] = acc.reg_expires;
		root["Accounts"][i]["stun_server"] = acc.stun_server;
	}

	// write contacts
	for (unsigned int i=0; i<uaConf.contacts.size(); i++)
	{
		UaConf::Contact &contact = uaConf.contacts[i];
		root["Contacts"][i]["description"] = contact.description;
		root["Contacts"][i]["user"] = contact.user;
		root["Contacts"][i]["sub_dialog_info"] = contact.sub_dialog_info;
	}

	std::string outputConfig = writer.write( root );

	try
	{
		std::ofstream ofs(asFileName.c_str());
		ofs << outputConfig;
		ofs.close();
	}
	catch(...)
	{
    	return 1;
	}

	return 0;
}
